@@ -71,6 +71,7 @@ public class JavacTaskImpl extends JavacTask {
     private JavaCompiler compiler;
     private Locale locale;
     private String[] args;
+    private String[] classNames;
     private Context context;
     private List<JavaFileObject> fileObjects;
     private Map<JavaFileObject, JCCompilationUnit> notYetEntered;
@@ -83,11 +84,13 @@ public class JavacTaskImpl extends JavacTask {
 
     JavacTaskImpl(Main compilerMain,
                 String[] args,
+                String[] classNames,
                 Context context,
                 List<JavaFileObject> fileObjects) {
         this.ccw = ClientCodeWrapper.instance(context);
         this.compilerMain = compilerMain;
         this.args = args;
+        this.classNames = classNames;
         this.context = context;
         this.fileObjects = fileObjects;
         setLocale(Locale.getDefault());
@@ -102,7 +105,7 @@ public class JavacTaskImpl extends JavacTask {
                 Context context,
                 Iterable<String> classes,
                 Iterable<? extends JavaFileObject> fileObjects) {
-        this(compilerMain, toArray(flags, classes), context, toList(fileObjects));
+        this(compilerMain, toArray(flags), toArray(classes), context, toList(fileObjects));
     }
 
     JavacTaskImpl(Context context) {
@@ -110,14 +113,11 @@ public class JavacTaskImpl extends JavacTask {
         setLocale(Locale.getDefault());
     }
 
-    static private String[] toArray(Iterable<String> flags, Iterable<String> classes) {
+    static private String[] toArray(Iterable<String> iter) {
         ListBuffer<String> result = new ListBuffer<String>();
-        if (flags != null)
-            for (String flag : flags)
-                result.append(flag);
-        if (classes != null)
-            for (String cls : classes)
-                result.append(cls);
+        if (iter != null)
+            for (String s : iter)
+                result.append(s);
         return result.toArray(new String[result.length()]);
     }
 
@@ -135,7 +135,7 @@ public class JavacTaskImpl extends JavacTask {
             initContext();
             notYetEntered = new HashMap<JavaFileObject, JCCompilationUnit>();
             compilerMain.setAPIMode(true);
-            result = compilerMain.compile(args, context, fileObjects, processors);
+            result = compilerMain.compile(args, classNames, context, fileObjects, processors);
             cleanup();
             return result == 0;
         } else {
@@ -168,10 +168,10 @@ public class JavacTaskImpl extends JavacTask {
         } else {
             initContext();
             compilerMain.setOptions(Options.instance(context));
-            compilerMain.filenames = new ListBuffer<File>();
-            List<File> filenames = compilerMain.processArgs(CommandLine.parse(args));
+            compilerMain.filenames = new LinkedHashSet<File>();
+            Collection<File> filenames = compilerMain.processArgs(CommandLine.parse(args), classNames);
             if (!filenames.isEmpty())
-                throw new IllegalArgumentException("Malformed arguments " + filenames.toString(" "));
+                throw new IllegalArgumentException("Malformed arguments " + toString(filenames, " "));
             compiler = JavaCompiler.instance(context);
             // NOTE: this value will be updated after annotation processing
             compiler.initProcessAnnotations(processors);
@@ -183,9 +183,21 @@ public class JavacTaskImpl extends JavacTask {
             // endContext will be called when all classes have been generated
             // TODO: should handle the case after each phase if errors have occurred
             args = null;
+            classNames = null;
         }
     }
 
+    <T> String toString(Iterable<T> items, String sep) {
+        String currSep = "";
+        StringBuilder sb = new StringBuilder();
+        for (T item: items) {
+            sb.append(currSep);
+            sb.append(item.toString());
+            currSep = sep;
+        }
+        return sb.toString();
+    }
+
     private void initContext() {
         context.put(JavacTaskImpl.class, this);
         if (context.get(TaskListener.class) != null)
@@ -202,6 +214,7 @@ public class JavacTaskImpl extends JavacTask {
         compiler = null;
         compilerMain = null;
         args = null;
+        classNames = null;
         context = null;
         fileObjects = null;
         notYetEntered = null;
@@ -363,8 +376,6 @@ public class JavacTaskImpl extends JavacTask {
                     compiler.skipAnnotationProcessing = false;
                 }
             }
-            compiler.flushTempDiags();
-
             ListBuffer<TypeElement> elements = new ListBuffer<TypeElement>();
             for (JCCompilationUnit unit : units) {
                 boolean isPkgInfo = unit.sourcefile.isNameCompatible("package-info",
@@ -463,9 +474,8 @@ public class JavacTaskImpl extends JavacTask {
                 };
                 f.run(compiler.todo, classes);
             }
-            if (!compiler.skipAnnotationProcessing)
+            if (!compiler.skipAnnotationProcessing && compiler.toProcessAnnotations.nonEmpty())
                 compiler = compiler.processAnnotations(List.<JCCompilationUnit>nil());
-            compiler.flushTempDiags();
         } finally {
             compiler.log.flush();
         }
@@ -808,65 +818,6 @@ public class JavacTaskImpl extends JavacTask {
         }
     }
 
-    public JCBlock reparseMethodBody(CompilationUnitTree topLevel, MethodTree methodToReparse, String newBodyText, int annonIndex,
-            final Map<? super JCTree,? super String> docComments) {
-        ParserFactory parserFactory = ParserFactory.instance(context);
-        CharBuffer buf = CharBuffer.wrap((newBodyText+"\u0000").toCharArray(), 0, newBodyText.length());
-        JavacParser parser = (JavacParser) parserFactory.newParser(buf, ((JCBlock)methodToReparse.getBody()).pos, ((JCCompilationUnit)topLevel).endPositions);
-        final JCStatement statement = parser.parseStatement();
-        JavacParser.assignAnonymousClassIndices(Names.instance(context), statement, Names.instance(context).empty, annonIndex);
-        if (statement.getKind() == Tree.Kind.BLOCK) {
-            if (docComments != null) {
-                docComments.putAll(parser.getDocComments());
-            }
-            return (JCBlock) statement;            
-        }
-        return null;
-    }
-
-    public BlockTree reattrMethodBody(MethodTree methodToReparse, BlockTree block) {
-        Attr attr = Attr.instance(context);
-        assert ((JCMethodDecl)methodToReparse).localEnv != null;
-        JCMethodDecl tree = (JCMethodDecl) methodToReparse;
-        final Names names = Names.instance(context);
-        final Symtab syms = Symtab.instance(context);
-        final MemberEnter memberEnter = MemberEnter.instance(context);
-        final Log log = Log.instance(context);
-        final TreeMaker make = TreeMaker.instance(context);
-        final Env<AttrContext> env = attr.dupLocalEnv(((JCMethodDecl) methodToReparse).localEnv);
-        final ClassSymbol owner = env.enclClass.sym;
-        if (tree.name == names.init && !owner.type.isErroneous() && owner.type != syms.objectType) {
-            JCBlock body = tree.body;
-            if (body.stats.isEmpty() || !TreeInfo.isSelfCall(body.stats.head)) {
-                body.stats = body.stats.
-                prepend(memberEnter.SuperCall(make.at(body.pos),
-                    List.<Type>nil(),
-                    List.<JCVariableDecl>nil(),
-                    false));
-            } else if ((env.enclClass.sym.flags() & Flags.ENUM) != 0 &&
-                (tree.mods.flags & Flags.GENERATEDCONSTR) == 0 &&
-                TreeInfo.isSuperCall(body.stats.head)) {
-                // enum constructors are not allowed to call super
-                // directly, so make sure there aren't any super calls
-                // in enum constructors, except in the compiler
-                // generated one.
-                log.error(tree.body.stats.head.pos(),
-                          "call.to.super.not.allowed.in.enum.ctor",
-                          env.enclClass.sym);
-                    }
-                }
-        attr.attribStat((JCBlock)block, env);
-        return block;
-    }
-
-    public BlockTree reflowMethodBody(CompilationUnitTree topLevel, ClassTree ownerClass, MethodTree methodToReparse) {
-        Flow flow = Flow.instance(context);
-        TreeMaker make = TreeMaker.instance(context);
-        flow.reanalyzeMethod(make.forToplevel((JCCompilationUnit)topLevel),
-                (JCClassDecl)ownerClass);
-        return methodToReparse.getBody();
-    }
-
     //Debug methods
     public String dumpTodo () {
         StringBuilder res = new StringBuilder ();
