@@ -150,7 +150,8 @@ public abstract class Symbol implements Element {
      * the default package; otherwise, the owner symbol is returned
      */
     public Symbol location() {
-        if (owner.name == null || (owner.name.isEmpty() && owner.kind != PCK && owner.kind != TYP)) {
+        if (owner.name == null || (owner.name.isEmpty() &&
+                (owner.flags() & BLOCK) == 0 && owner.kind != PCK && owner.kind != TYP)) {
             return null;
         }
         return owner;
@@ -181,7 +182,7 @@ public abstract class Symbol implements Element {
      */
     public Type externalType(Types types) {
         Type t = erasure(types);
-        if (name == name.table.names.init && owner.hasOuterInstance()) {
+        if (name != null && name == name.table.names.init && owner != null && owner.hasOuterInstance()) {
             Type outerThisType = types.erasure(owner.type.getEnclosingType());
             return new MethodType(t.getParameterTypes().prepend(outerThisType),
                                   t.getReturnType(),
@@ -261,7 +262,7 @@ public abstract class Symbol implements Element {
     /** A class is an inner class if it it has an enclosing instance class.
      */
     public boolean isInner() {
-        return type.getEnclosingType().tag == CLASS;
+        return type.getEnclosingType() != null && type.getEnclosingType().tag == CLASS;
     }
 
     /** An inner class has an outer instance if it is not an interface
@@ -273,7 +274,7 @@ public abstract class Symbol implements Element {
      *  @see #isInner
      */
     public boolean hasOuterInstance() {
-        return
+        return type.getEnclosingType() != null &&
             type.getEnclosingType().tag == CLASS && (flags() & (INTERFACE | NOOUTERTHIS)) == 0;
     }
 
@@ -578,7 +579,7 @@ public abstract class Symbol implements Element {
 
         public java.util.List<Symbol> getEnclosedElements() {
             List<Symbol> list = List.nil();
-            if (kind == TYP && type.tag == TYPEVAR) {
+            if (kind == NIL || (kind == TYP && type.tag == TYPEVAR || type.tag == UNKNOWN)) {
                 return list;
             }
             for (Scope.Entry e = members().elems; e != null; e = e.sibling) {
@@ -731,14 +732,15 @@ public abstract class Symbol implements Element {
          */
         public JavaFileObject classfile;
 
+        /** the list of translated local classes (used for generating
+         * InnerClasses attribute)
+         */
+        public List<ClassSymbol> trans_local;
+
         /** the constant pool of the class
          */
         public Pool pool;
 
-        /** members closure cache (set by Types.membersClosure)
-         */
-        Scope.CompoundScope membersClosure;
-
         public ClassSymbol(long flags, Name name, Type type, Symbol owner) {
             super(flags, name, type, owner);
             this.members_field = null;
@@ -779,7 +781,9 @@ public abstract class Symbol implements Element {
         }
 
         public List<Attribute.Compound> getAnnotationMirrors() {
-            if (completer != null) complete();
+            try {
+                if (completer != null) complete();
+            } catch (CompletionFailure cf) {}
             return Assert.checkNonNull(attributes_field);
         }
 
@@ -1322,7 +1326,7 @@ public abstract class Symbol implements Element {
             // if origin is derived from a raw type, we might have missed
             // an implementation because we do not know enough about instantiations.
             // in this case continue with the supertype as origin.
-            if (types.isDerivedRaw(origin.type))
+            if (types.isDerivedRaw(origin.type) && !origin.isInterface())
                 return implementation(types.supertype(origin.type).tsym, types, checkResult);
             else
                 return null;
@@ -1393,12 +1397,17 @@ public abstract class Symbol implements Element {
                 return ElementKind.CONSTRUCTOR;
             else if (name == name.table.names.clinit)
                 return ElementKind.STATIC_INIT;
-            else if ((flags_field & BLOCK) != 0)
-                return (flags_field & STATIC) != 0 ? ElementKind.STATIC_INIT : ElementKind.INSTANCE_INIT;
+            else if ((flags() & BLOCK) != 0)
+                return isStatic() ? ElementKind.STATIC_INIT : ElementKind.INSTANCE_INIT;
             else
                 return ElementKind.METHOD;
         }
 
+        public boolean isStaticOrInstanceInit() {
+            return getKind() == ElementKind.STATIC_INIT ||
+                    getKind() == ElementKind.INSTANCE_INIT;
+        }
+
         public Attribute getDefaultValue() {
             return defaultValue;
         }
