@@ -113,8 +113,6 @@ public class Enter extends JCTree.Visitor {
     JavaFileManager fileManager;
     PkgInfo pkginfoOpt;
     
-    private final CancelService cancelService;
-    private final LowMemoryWatch memoryWatch;
     private final LazyTreeLoader treeLoader;
     private final DuplicateClassChecker duplicateClassChecker;
     private final Source source;
@@ -141,8 +139,6 @@ public class Enter extends JCTree.Visitor {
         annotate = Annotate.instance(context);
         lint = Lint.instance(context);
         names = Names.instance(context);
-        cancelService = CancelService.instance(context);
-        memoryWatch = LowMemoryWatch.instance(context);
         treeLoader = LazyTreeLoader.instance(context);
         duplicateClassChecker = context.get(DuplicateClassChecker.class);
 
@@ -267,8 +263,12 @@ public class Enter extends JCTree.Visitor {
             assert typeEnvsShadow == null;
             typeEnvsShadow = new HashMap<TypeSymbol,Env<AttrContext>>();
         } else {
-            for (Map.Entry<TypeSymbol, Env<AttrContext>> entry : typeEnvsShadow.entrySet())
-                typeEnvs.put(entry.getKey(), entry.getValue());
+            for (Map.Entry<TypeSymbol, Env<AttrContext>> entry : typeEnvsShadow.entrySet()) {
+                if (entry.getValue() == null)
+                    typeEnvs.remove(entry.getKey());
+                else
+                    typeEnvs.put(entry.getKey(), entry.getValue());
+            }
             typeEnvsShadow = null;
         }
     }
@@ -312,7 +312,6 @@ public class Enter extends JCTree.Visitor {
             Type t = classEnter(l.head, env);
             if (t != null)
                 ts.append(t);
-            memoryWatch.abortIfMemoryLow();
         }
         return ts.toList();
     }
@@ -406,7 +405,6 @@ public class Enter extends JCTree.Visitor {
 
     @Override
     public void visitClassDef(JCClassDecl tree) {
-        cancelService.abortIfCanceled();
         Symbol owner = env.info.scope.owner;
         Scope enclScope = enterScope(env);
         ClassSymbol c = null;
@@ -476,13 +474,13 @@ public class Enter extends JCTree.Visitor {
                     }
                 } else {
                     // We are seeing a local class.
-                    if (tree.index == -1) {
+                    if (getIndex(tree) == -1) {
                         c = reader.defineClass(tree.name, owner);
                         c.flatname = chk.localClassName(c);
                         noctx = true;
                     }
                     else {
-                        Name flatname = chk.localClassName(owner.enclClass(), tree.name, tree.index);
+                        Name flatname = chk.localClassName(owner.enclClass(), tree.name, getIndex(tree));
                         if ((c=chk.compiled.get(flatname)) != null) {
                             reattr = true;
                         }
@@ -576,6 +574,20 @@ public class Enter extends JCTree.Visitor {
         } else {
             c.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, c, tree) | (c.flags_field & (FROMCLASS | APT_CLEANED));
             ClassType ct = (ClassType)c.type;
+            if (owner.kind != PCK && (c.flags_field & STATIC) == 0) {
+                // We are seeing a local or inner class.
+                // Set outer_field of this class to closest enclosing class
+                // which contains this class in a non-static context
+                // (its "enclosing instance class"), provided such a class exists.
+                Symbol owner1 = owner;
+                while ((owner1.kind & (VAR | MTH)) != 0 &&
+                        (owner1.flags_field & STATIC) == 0) {
+                    owner1 = owner1.owner;
+                }
+                if (owner1.kind == TYP) {
+                    ct.setEnclosingType(owner1.type);
+                }
+            }
             boolean wasNull = false;
             if (ct.typarams_field != null) {
                 for (List<Type> l = ct.typarams_field; l.nonEmpty(); l = l.tail)
@@ -631,6 +643,10 @@ public class Enter extends JCTree.Visitor {
         log.error(pos, "duplicate.class", c.fullname);
     }
 
+    protected int getIndex(JCClassDecl clazz) {
+        return -1;
+    }
+
     /** Class enter visitor method for type parameters.
      *  Enter a symbol for type parameter in local scope, after checking that it
      *  is unique.
@@ -700,7 +716,6 @@ public class Enter extends JCTree.Visitor {
                     else
                         // defer
                         prevUncompleted.append(clazz);
-                    memoryWatch.abortIfMemoryLow();
                 }
 
                 // if there remain any unimported toplevels (these must have
@@ -711,7 +726,6 @@ public class Enter extends JCTree.Visitor {
                         Env<AttrContext> topEnv = topLevelEnv(tree);
                         memberEnter.memberEnter(tree, topEnv);
                         log.useSource(prev);
-                        memoryWatch.abortIfMemoryLow();
                     }
                 }
             }
