@@ -344,10 +344,12 @@ public class Flow extends TreeScanner {
      *  is caught.
      */
     void markThrown(JCTree tree, Type exc) {
-        if (!chk.isUnchecked(tree.pos(), exc)) {
-            if (!chk.isHandled(exc, caught))
-                pendingExits.append(new PendingExit(tree, exc));
-                thrown = chk.incl(exc, thrown);
+        if (exc != syms.unknownType) {
+            if (!chk.isUnchecked(tree.pos(), exc)) {
+                if (!chk.isHandled(exc, caught))
+                    pendingExits.append(new PendingExit(tree, exc));
+                    thrown = chk.incl(exc, thrown);
+            }
         }
     }
 
@@ -770,7 +772,7 @@ public class Flow extends TreeScanner {
             alive = true;
             scanStat(tree.body);
 
-            if (alive && tree.sym.type.getReturnType().tag != VOID)
+            if (alive && tree.sym.type.getReturnType() != null && tree.sym.type.getReturnType().tag != VOID)
                 log.error(TreeInfo.diagEndPos(tree.body), "missing.ret.stmt");
 
             if (isInitialConstructor) {
@@ -806,6 +808,10 @@ public class Flow extends TreeScanner {
 
     public void visitVarDef(JCVariableDecl tree) {
         if (tree.sym == null) {
+            if (tree.type == null)
+                tree.type = syms.unknownType;
+            tree.sym = new VarSymbol(0, tree.name, tree.type, syms.noSymbol);
+            tree.sym.adr = 0;
             String variableName = String.valueOf(tree.name);
             int variableStart = tree.getStartPosition();
 
