@@ -42,8 +42,9 @@ import com.sun.tools.javac.tree.JCTree.*;
 import static com.sun.tools.javac.code.Flags.*;
 import static com.sun.tools.javac.code.Kinds.*;
 import static com.sun.tools.javac.code.TypeTags.*;
-import com.sun.tools.javac.model.LazyTreeLoader;
+import com.sun.tools.javac.util.JCDiagnostic.DiagnosticFlag;
 import com.sun.tools.javac.util.JCDiagnostic.DiagnosticPosition;
+import com.sun.tools.javac.model.LazyTreeLoader;
 import javax.lang.model.element.ElementKind;
 import javax.lang.model.element.Modifier;
 
@@ -79,7 +80,6 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
     private final Target target;
     private final DeferredLintHandler deferredLintHandler;
     private final JavacMessages messages;
-    private final CancelService cancelService;
     private final LazyTreeLoader treeLoader;
 
     private final boolean skipAnnotations;
@@ -114,7 +114,6 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         boolean backgroundCompilation = options.get("backgroundCompilation") != null;
         ignoreNoLang = ideMode && !backgroundCompilation;
         messages = JavacMessages.instance(context);
-        cancelService = CancelService.instance(context);
         treeLoader = LazyTreeLoader.instance(context);
     }
 
@@ -158,7 +157,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                     throw new MissingPlatformError (msg);
                 }
             } else {
-                log.error(pos, "doesnt.exist", tsym);
+                log.error(DiagnosticFlag.RESOLVE_ERROR, pos, "doesnt.exist", tsym);
             }
         }
         env.toplevel.starImportScope.importAll(tsym.members());
@@ -267,7 +266,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                                    final Name name,
                                    final Env<AttrContext> env) {
         if (tsym.kind != TYP) {
-            log.error(pos, "static.imp.only.classes.and.interfaces");
+            log.error(DiagnosticFlag.RECOVERABLE, pos, "static.imp.only.classes.and.interfaces");
             return;
         }
 
@@ -522,7 +521,6 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
     }
 
     public void visitTopLevel(JCCompilationUnit tree) {
-        cancelService.abortIfCanceled();
         if (tree.starImportScope.elems != null) {
             // we must have already processed this toplevel
             return;
@@ -555,7 +553,6 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
 
     // process the non-static imports and the static imports of types.
     public void visitImport(JCImport tree) {
-        cancelService.abortIfCanceled();
         JCTree imp = tree.qualid;
         Name name = TreeInfo.name(imp);
         TypeSymbol p;
@@ -591,8 +588,60 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         }
     }
 
+    boolean hasSameBounds(ForAll t, ForAll s) {
+        List<Type> l1 = t.tvars;
+        List<Type> l2 = s.tvars;
+        while (l1.nonEmpty() && l2.nonEmpty() &&
+               isSameAPType(l1.head.getUpperBound(),
+                          types.subst(l2.head.getUpperBound(),
+                                s.tvars,
+                                t.tvars))) {
+            l1 = l1.tail;
+            l2 = l2.tail;
+        }
+        return l1.isEmpty() && l2.isEmpty();
+    }
+    
+    private boolean isSameAPType(Type t, Type s) {
+        return    types.isSameType(t, s)
+               || t.tsym != null && s.tsym != null && s.isErroneous() && t.tsym.name == s.tsym.name;
+    }
+    
+    private boolean isSameMethod(Type ot, Type os) {
+        if (ot.tag != os.tag) return false;
+        
+        switch (ot.tag) {
+            case METHOD: {
+                MethodType t = (MethodType) ot;
+                MethodType s = (MethodType) os;
+
+                if (t.getParameterTypes().size() != s.getParameterTypes().size()) return false;
+                if (!isSameAPType(t.getReturnType(), s.getReturnType())) return false;
+
+                List<Type> tp = t.getParameterTypes();
+                List<Type> sp = s.getParameterTypes();
+
+                while (!tp.isEmpty() && !sp.isEmpty()) {
+                    if (!isSameAPType(tp.head, sp.head)) return false;
+                    tp = tp.tail;
+                    sp = sp.tail;
+                }
+
+                return tp.isEmpty() == sp.isEmpty();
+            }
+            case FORALL: {
+                ForAll t = (ForAll) ot;
+                ForAll s = (ForAll) os;
+
+                return    hasSameBounds(t, s)
+                       && isSameMethod(t.qtype, types.subst(s.qtype, s.tvars, t.tvars));
+            }
+            default:
+                return false;
+        }
+    }
+    
     public void visitMethodDef(JCMethodDecl tree) {
-        cancelService.abortIfCanceled();
         Scope enclScope = enter.enterScope(env);
         MethodSymbol m = new MethodSymbol(0, tree.name, null, enclScope.owner);
         m.flags_field = chk.checkFlags(tree.pos(), tree.mods.flags, m, tree);
@@ -613,7 +662,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         if ((enclScope.owner.flags_field & FROMCLASS) != 0) {
             for (Scope.Entry e = enclScope.lookup(tree.name); e.scope == enclScope; e = e.next()) {
                 if (e.sym.kind == MTH) {
-                    boolean sameType = types.isSameType(m.type, e.sym.type);
+                    boolean sameType = (enclScope.owner.flags_field & APT_CLEANED) != 0 ? isSameMethod(m.type, e.sym.type) : types.isSameType(m.type, e.sym.type);
                     if (sameType || m.name == names.init && (m.owner.name.isEmpty() || (m.owner.owner.kind & (VAR | MTH)) != 0)) {
                         if ((e.sym.flags_field & FROMCLASS) != 0) {
                             treeCleaner.scan(tree);
@@ -625,16 +674,14 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                                 for(List<Type> tvars = ((ForAll)tree.sym.type).tvars; tvars.nonEmpty(); tvars = tvars.tail)
                                     localEnv.info.scope.enter(tvars.head.tsym);
                             }
-                            List<VarSymbol> p = tree.sym.params();
+                            List<VarSymbol> p = tree.sym.params().reverse();
                             if (p != null) {
-                                List<JCVariableDecl> l = tree.params;
+                                List<JCVariableDecl> l = tree.params.reverse();
                                 while(l.nonEmpty() && p.nonEmpty()) {
-                                    if (sameType) {
-                                        p.head.setName(l.head.name);
-                                        if (l.head.getModifiers() != null && l.head.getModifiers().getFlags().contains(Modifier.FINAL)) {
-                                            //copy the final flag, as the symbol might have come from the classfile:
-                                            p.head.flags_field |= FINAL;
-                                        }
+                                    p.head.setName(l.head.name);
+                                    if (l.head.getModifiers() != null && l.head.getModifiers().getFlags().contains(Modifier.FINAL)) {
+                                        //copy the final flag, as the symbol might have come from the classfile:
+                                        p.head.flags_field |= FINAL;
                                     }
                                     localEnv.info.scope.enter(p.head);
                                     p.head.flags_field |= FROMCLASS;
@@ -647,9 +694,15 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                                     p = p.tail;
                                 }
                             }
-                            tree.sym.type = signature(tree.typarams, tree.params,
-                                    tree.restype, tree.thrown,
-                                    localEnv);
+                            prevLintHandler =
+                                    chk.setDeferredLintHandler(deferredLintHandler.setPos(tree.pos()));
+                            try {
+                                tree.sym.type = signature(tree.typarams, tree.params,
+                                        tree.restype, tree.thrown,
+                                        localEnv);
+                            } finally {
+                                chk.setDeferredLintHandler(prevLintHandler);
+                            }
                             tree.sym.flags_field &= ~FROMCLASS;
                             localEnv.info.scope.leave();
 
@@ -714,9 +767,9 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
             if( chk.checkUnique(tree.pos(), m, enclScope))
                 enclScope.enter(m);
         }
-        annotateLater(tree.mods.annotations, localEnv, m);
+        annotateLater(tree.mods.annotations, localEnv, tree.sym);
         if (tree.defaultValue != null)
-            annotateDefaultValueLater(tree.defaultValue, localEnv, m);
+            annotateDefaultValueLater(tree.defaultValue, localEnv, tree.sym);
     }
 
     /** Create a fresh environment for method bodies.
@@ -733,7 +786,6 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
     }
 
     public void visitVarDef(JCVariableDecl tree) {
-        cancelService.abortIfCanceled();
         Env<AttrContext> localEnv = env;
         if ((tree.mods.flags & STATIC) != 0 ||
             (env.info.scope.owner.flags() & INTERFACE) != 0) {
@@ -761,9 +813,11 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         boolean doEnterSymbol = true;
         if ((enclScope.owner.flags_field & FROMCLASS) != 0) {
             for (Scope.Entry e = enclScope.lookup(tree.name); e.scope == enclScope; e = e.next()) {
-                if (e.sym.kind == VAR && types.isSameType(tree.vartype.type, e.sym.type)) {
+                boolean sameType = (enclScope.owner.flags_field & APT_CLEANED) != 0 ? isSameAPType(tree.vartype.type, e.sym.type) : types.isSameType(tree.vartype.type, e.sym.type);
+                if (e.sym.kind == VAR && sameType) {
                     if ((e.sym.flags_field & FROMCLASS) != 0) {
                         v = (VarSymbol)e.sym;
+                        v.type = tree.vartype.type;
                         e.sym.flags_field &= ~FROMCLASS;
                     }
                     break;
