@@ -36,6 +36,8 @@ import com.sun.tools.javac.code.Type;
 import com.sun.tools.javac.code.Type.ClassType;
 import com.sun.tools.javac.code.TypeTags;
 import com.sun.tools.javac.code.Types;
+import com.sun.tools.javac.jvm.Pool;
+import com.sun.tools.javac.parser.Token;
 import com.sun.tools.javac.tree.JCTree;
 import com.sun.tools.javac.tree.JCTree.JCAssignOp;
 import com.sun.tools.javac.tree.JCTree.JCBinary;
@@ -66,6 +68,8 @@ import com.sun.tools.javac.util.Name;
 import java.util.HashSet;
 import java.util.Set;
 import java.util.logging.Logger;
+import javax.lang.model.element.ElementKind;
+import javax.tools.JavaFileObject;
 
 /**
  *
@@ -107,6 +111,7 @@ public class Repair extends TreeTranslator {
     private Set<ClassSymbol> repairedClasses = new HashSet<ClassSymbol>();
     private boolean isErrClass;
     private boolean insideErrEnum;
+    private Name fixedTopLevelName;
     
     private Repair(Context context) {
         context.put(repairKey, this);
@@ -124,6 +129,9 @@ public class Repair extends TreeTranslator {
     public <T extends JCTree> T translate(T tree) {
         if (tree == null)
             return null;
+        if (tree.type != null && tree.type.constValue() instanceof String && ((String)tree.type.constValue()).length() >= Pool.MAX_STRING_LENGTH) {
+            log.error(tree.pos(), "limit.string"); //NOI18N
+        }
         parents = parents.prepend(tree);
         try {
             if (hasError)
@@ -134,7 +142,7 @@ public class Repair extends TreeTranslator {
         } finally {
             parents = parents.tail;            
         }
-        if (tree.type != null && tree.type.isErroneous()) {
+        if (tree.type != null && tree.type.isErroneous() || tree.type == syms.unknownType) {
             JCTree parent = parents.head;
             if (parent == null || parent.getTag() != JCTree.CLASSDEF) {
                 hasError = true;
@@ -176,7 +184,7 @@ public class Repair extends TreeTranslator {
         super.visitTypeParameter(tree);
         if (tree.type != null && tree.type.tag == TypeTags.TYPEVAR) {
             Type.TypeVar tv = (Type.TypeVar)tree.type;
-            if (tv.bound != null && tv.bound.isErroneous()) {
+            if (tv.bound != null && tv.bound.isErroneous() || tv.bound == syms.unknownType) {
                 if (err == null && errMessage == null)
                     errMessage = "Erroneous type var bound: " + tv.bound;
                 tv.bound = syms.objectType;
@@ -205,6 +213,8 @@ public class Repair extends TreeTranslator {
                 hasError = false;
                 err = null;
                 errMessage = null;
+                if (tree.sym != null)
+                    tree.sym.setData(null);
             }
         } else if (tree.sym == null) {
             JCTree parent = parents != null ? parents.tail.head : null;
@@ -282,7 +292,7 @@ public class Repair extends TreeTranslator {
             hasError = true;
             if (err == null && errMessage == null)
                 errMessage = "Null sym type: " + meth;
-        } else if (meth.type.isErroneous()) {
+        } else if (meth.type.isErroneous() || meth.type == syms.unknownType) {
             hasError = true;
             if (err == null && errMessage == null)
                 errMessage = "Erroneous sym type: " + meth.type;
@@ -302,7 +312,7 @@ public class Repair extends TreeTranslator {
             hasError = true;
             if (err == null && errMessage == null)
                 errMessage = "Null ctor sym type: " + ctor;
-        } else if (tree.constructorType.isErroneous()) {
+        } else if (tree.constructorType.isErroneous() || tree.constructorType == syms.unknownType) {
             hasError = true;
             if (err == null && errMessage == null)
                 errMessage = "Erroneous ctor sym type: " + tree.constructorType;
@@ -408,7 +418,7 @@ public class Repair extends TreeTranslator {
         if (c == null)
             return;
         Type st = types.supertype(c.type);
-        if (st.tag == TypeTags.CLASS)
+        if (st != null && st.tag == TypeTags.CLASS)
             translateClass((ClassSymbol)st.tsym);
         LOGGER.finest("Repair.translateClass: " + c); //NOI18N
         if (repairedClasses.contains(c)) {
@@ -441,10 +451,33 @@ public class Repair extends TreeTranslator {
                         break;
                 }
                 hasError = false;
-                isErrClass = c.type.isErroneous();
+                isErrClass = c.type.isErroneous() || c.type == syms.unknownType;
                 err = null;
                 staticInit = null;
                 JCClassDecl tree = (JCClassDecl)attrEnv.tree;
+                final Symbol enclosingElement = c.getEnclosingElement();
+                if (c.name == c.name.table.names.error &&
+                    enclosingElement.getKind() == ElementKind.PACKAGE) {
+                    final JavaFileObject source = c.sourcefile;
+                    final String path = source.toUri().getPath();
+                    int start = path.lastIndexOf('/');
+                    int end = path.lastIndexOf('.');
+                    fixedTopLevelName = c.name.table.fromString(path.substring(start+1, end));
+                    c.name = fixedTopLevelName;
+                    c.fullname = Symbol.TypeSymbol.formFullName(c.name, enclosingElement);
+                    c.flatname = c.fullname;
+                    tree.name = c.name;
+                    isErrClass = true;
+                    hasError = true;
+                    err = diags.error(
+                            log.currentSource(),
+                            tree,
+                            "expected",         //NOI18N
+                            Token.IDENTIFIER);
+                } else if (fixedTopLevelName != null) {
+                    c.fullname = Symbol.TypeSymbol.formFullName(c.name, enclosingElement);
+                    c.flatname = Symbol.TypeSymbol.formFlatName(c.name, enclosingElement);
+                }
                 tree.mods = translate(tree.mods);
                 tree.typarams = translateTypeParams(tree.typarams);
                 tree.extending = translate(tree.extending);
@@ -466,7 +499,7 @@ public class Repair extends TreeTranslator {
                     isErrClass = true;
                     classLevelErrTree = err.getTree();
                     classLevelErrMessage = err.getMessage(null);
-                } else if (c.type.isErroneous() && oldHasError && oldErr != null) {
+                } else if ((c.type.isErroneous() || c.type == syms.unknownType) && oldHasError && oldErr != null) {
                     classLevelErrTree = oldErr.getTree();
                     classLevelErrMessage = oldErr.getMessage(null);
                 }
@@ -482,30 +515,51 @@ public class Repair extends TreeTranslator {
                             nonAbstractMethods.remove(((JCMethodDecl)l.head).sym);
                         hasError = false;
                         err = null;
-                        l.head = translate(l.head);
-                        if ((l.head.getTag() == JCTree.METHODDEF && ((JCMethodDecl)l.head).sym == null)
-                                || (l.head.getTag() == JCTree.VARDEF && ((JCVariableDecl)l.head).sym == null)) {
-                            hasError = true;
-                        }
-                        if (hasError) {
-                            if (l.head.getTag() == JCTree.CLASSDEF && tree.sym.members_field.includes(((JCClassDecl)l.head).sym)) {
-                                last = l;
-                            } else {
-                                if (last != null)
-                                    last.tail = l.tail;
-                                else
-                                    tree.defs = l.tail;
-                            }
-                            if (classLevelErrTree == null) {
-                                if (err != null) {
-                                    classLevelErrTree = err.getTree();
-                                    classLevelErrMessage = err.getMessage(null);
+                        if (l.head.getTag() == JCTree.CLASSDEF && ((JCClassDecl)l.head).name == c.name.table.names.error) {
+                            Scope.Entry prev = null;
+                            for (Scope.Entry e = tree.sym.members_field.elems; e != null; e = e.sibling) {
+                                if (e.sym == ((JCClassDecl)l.head).sym) {
+                                    if (prev == null) {
+                                        tree.sym.members_field.elems = e.sibling;
+                                        break;
+                                    } else {
+                                        prev.sibling = e.sibling;
+                                        break;
+                                    }
                                 } else {
-                                    classLevelErrTree = l.head;
+                                    prev = e;      
                                 }
                             }
+                            if (last != null)
+                                last.tail = l.tail;
+                            else
+                                tree.defs = l.tail;
                         } else {
-                            last = l;
+                            l.head = translate(l.head);
+                            if ((l.head.getTag() == JCTree.METHODDEF && ((JCMethodDecl)l.head).sym == null)
+                                    || (l.head.getTag() == JCTree.VARDEF && ((JCVariableDecl)l.head).sym == null)) {
+                                hasError = true;
+                            }
+                            if (hasError) {
+                                if (l.head.getTag() == JCTree.CLASSDEF && tree.sym.members_field.includes(((JCClassDecl)l.head).sym)) {
+                                    last = l;
+                                } else {
+                                    if (last != null)
+                                        last.tail = l.tail;
+                                    else
+                                        tree.defs = l.tail;
+                                }
+                                if (classLevelErrTree == null) {
+                                    if (err != null) {
+                                        classLevelErrTree = err.getTree();
+                                        classLevelErrMessage = err.getMessage(null);
+                                    } else {
+                                        classLevelErrTree = l.head;
+                                    }
+                                }
+                            } else {
+                                last = l;
+                            }
                         }
                     }
                     if (classLevelErrTree != null) {
@@ -515,11 +569,14 @@ public class Repair extends TreeTranslator {
                             tree.defs = tree.defs.prepend(generateErrStaticInit(classLevelErrTree, classLevelErrMessage));
                     }
                     for (MethodSymbol symbol : nonAbstractMethods) {
+                        if ((symbol.flags() & Flags.BRIDGE) != 0) {
+                            continue;
+                        }
                         if ((symbol.owner.flags() & Flags.ENUM) != 0) {
                             if ((symbol.name == symbol.name.table.names.values
                                     && symbol.type.asMethodType().argtypes.isEmpty())
                                     || (symbol.name == symbol.name.table.names.valueOf
-                                    && symbol.type.asMethodType().argtypes.head == enter.syms.stringType
+                                    && types.isSameType(symbol.type.asMethodType().argtypes.head, enter.syms.stringType)
                                     && symbol.type.asMethodType().argtypes.tail.isEmpty())) {
                                 continue;
                             }
@@ -553,6 +610,7 @@ public class Repair extends TreeTranslator {
         } finally {
             attrEnv = null;
             make = null;
+            fixedTopLevelName = null;
         }
     }
     
