@@ -26,7 +26,6 @@
 package com.sun.tools.javac.comp;
 
 import java.util.*;
-import java.util.logging.Logger;
 
 import javax.lang.model.element.ElementKind;
 
@@ -108,7 +107,7 @@ public class TransTypes extends TreeTranslator {
         make.at(tree.pos);
         if (!types.isSameType(tree.type, target)) {
             if (!resolve.isAccessible(env, target.tsym))
-                resolve.logAccessError(env, tree, target);
+                resolve.logAccessErrorInternal(env, tree, target);
             tree = make.TypeCast(make.Type(target), tree).setType(target);
         }
         make.pos = oldpos;
@@ -154,7 +153,7 @@ public class TransTypes extends TreeTranslator {
      */
     JCExpression retype(JCExpression tree, Type erasedType, Type target) {
 //      System.err.println("retype " + tree + " to " + erasedType);//DEBUG
-        if (erasedType != null && erasedType.tag > lastBaseTag) {
+        if (erasedType.tag > lastBaseTag) {
             if (target != null && target.isPrimitive())
                 target = erasure(tree.type);
             tree.type = erasedType;
@@ -340,6 +339,8 @@ public class TransTypes extends TreeTranslator {
         private boolean isBridgeNeeded(MethodSymbol method,
                                        MethodSymbol impl,
                                        Type dest) {
+            if (method.type.isErroneous())
+                return false;
             if (impl != method) {
                 // If either method or impl have different erasures as
                 // members of dest, a bridge is needed.
@@ -409,21 +410,6 @@ public class TransTypes extends TreeTranslator {
             addBridges(pos, l.head.tsym, origin, bridges);
     }
 
-
-    public void getBridges (DiagnosticPosition pos, ClassSymbol origin, ListBuffer<JCTree> bridges) {
-
-        Env<AttrContext> myEnv = enter.typeEnvs.get(origin);
-        if (myEnv == null)
-            return;
-        Env<AttrContext> oldEnv = env;
-        try {
-            env = myEnv;
-            addBridges (pos, origin, bridges);
-        } finally {
-            env = oldEnv;
-        }
-    }
-
 /* ************************************************************************
  * Visitor methods
  *************************************************************************/
@@ -483,7 +469,7 @@ public class TransTypes extends TreeTranslator {
         for (Scope.Entry e = tree.sym.owner.members().lookup(tree.name);
              e.sym != null;
              e = e.next()) {
-            if (e.sym != tree.sym && tree.type != null &&
+            if (e.sym != tree.sym &&
                 types.isSameType(erasure(e.sym.type), tree.type)) {
                 log.error(tree.pos(),
                           "name.clash.same.erasure", tree.sym,
@@ -610,8 +596,7 @@ public class TransTypes extends TreeTranslator {
         if (tree.varargsElement != null)
             tree.varargsElement = types.erasure(tree.varargsElement);
         else
-            if (!(tree.args.length() == argtypes.length()))
-                Assert.error("[" + meth.owner + "]'s method [" + meth + "] of type: [" + mt + "]; has different number of parameters than tree [" + tree + "].\nEnv tree: [" + env.tree + "]"); //NOI18N
+            Assert.check(tree.args.length() == argtypes.length());
         tree.args = translateArgs(tree.args, argtypes, tree.varargsElement);
 
         // Insert casts of method invocation results as needed.
@@ -659,24 +644,19 @@ public class TransTypes extends TreeTranslator {
 
     public void visitAssignop(JCAssignOp tree) {
         tree.lhs = translate(tree.lhs, null);
-        if (tree.operator != null)
-            tree.rhs = translate(tree.rhs, tree.operator.type.getParameterTypes().tail.head);
+        tree.rhs = translate(tree.rhs, tree.operator.type.getParameterTypes().tail.head);
         tree.type = erasure(tree.type);
         result = tree;
     }
 
     public void visitUnary(JCUnary tree) {
-        if (tree.operator != null) {
-            tree.arg = translate(tree.arg, tree.operator.type.getParameterTypes().head);
-        }
+        tree.arg = translate(tree.arg, tree.operator.type.getParameterTypes().head);
         result = tree;
     }
 
     public void visitBinary(JCBinary tree) {
-        if (tree.operator != null) {
-            tree.lhs = translate(tree.lhs, tree.operator.type.getParameterTypes().head);
-            tree.rhs = translate(tree.rhs, tree.operator.type.getParameterTypes().tail.head);
-        }
+        tree.lhs = translate(tree.lhs, tree.operator.type.getParameterTypes().head);
+        tree.rhs = translate(tree.rhs, tree.operator.type.getParameterTypes().tail.head);
         result = tree;
     }
 
@@ -875,8 +855,6 @@ public class TransTypes extends TreeTranslator {
         if (st.tag == CLASS)
             translateClass((ClassSymbol)st.tsym);
 
-        if ((c.flags_field & UNATTRIBUTED) != 0)
-            Logger.getLogger(TransTypes.class.getName()).warning("TransTypes.translateClass removes unattributed class: [" + c + "] form the typeEnvs map."); //NOI18N
         Env<AttrContext> myEnv = enter.typeEnvs.remove(c);
         if (myEnv == null)
             return;
