@@ -42,7 +42,6 @@ import java.util.logging.Logger;
 
 import javax.annotation.processing.Processor;
 import javax.lang.model.SourceVersion;
-import javax.tools.Diagnostic;
 import javax.tools.JavaFileManager;
 import javax.tools.JavaFileObject;
 import javax.tools.DiagnosticListener;
@@ -295,10 +294,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     protected ParserFactory parserFactory;
 
 
-    protected FlowListener flowListener;
-    
-    protected LowMemoryWatch memoryWatch;
-
     /**
      * Annotation processing may require and provide a new instance
      * of the compiler to be used for the analyze and generate phases.
@@ -348,8 +343,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         enter = Enter.instance(context);
         memberEnter = MemberEnter.instance(context);
         todo = Todo.instance(context);
-        flowListener = FlowListener.instance(context);
-        memoryWatch = LowMemoryWatch.instance(context);
 
         fileManager = context.get(JavaFileManager.class);
         parserFactory = ParserFactory.instance(context);
@@ -626,33 +619,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             return keepComments || sourceOutput || stubOutput;
         }
 
-    private List<Diagnostic<? extends JavaFileObject>> tempDiags = List.nil();
-    private Set<Pair<JavaFileObject, Integer>> recordedOverlay;
-    private Map<JCTree, JCDiagnostic> errTreesOverlay;
-    private DiagnosticListener<? super JavaFileObject> temporaryDiagListener = new DiagnosticListener<JavaFileObject>() {
-        public void report(Diagnostic<? extends JavaFileObject> diagnostic) {
-            tempDiags = tempDiags.prepend(diagnostic);
-        }
-    };
-    public void flushTempDiags() {
-        DiagnosticListener<? super JavaFileObject> diagnosticListener = log.getDiagnosticListener();
-        if (diagnosticListener != null) {
-            tempDiags = tempDiags.reverse();
-            for (Diagnostic<? extends JavaFileObject> diagnostic : tempDiags) {
-                diagnosticListener.report(diagnostic);
-            }
-        }
-        if (recordedOverlay != null) {
-            log.pushRecordedOverlay(recordedOverlay);
-            recordedOverlay = null;
-        }
-        if (errTreesOverlay != null) {
-            log.pushErrTreesOverlay(errTreesOverlay);
-            errTreesOverlay = null;
-        }
-        tempDiags = List.nil();
-    }
-
     /** Parse contents of file.
      *  @param filename     The name of the file to be parsed.
      */
@@ -808,7 +774,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                 } finally {
                     skipAnnotationProcessing = false;
                 }
-                flushTempDiags();
             }
         }
 
@@ -880,7 +845,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                     enterTrees(stopIfError(CompileState.PARSE, parseFiles(sourceFileObjects))),
                     classnames);
 
-            delegateCompiler.flushTempDiags();
             delegateCompiler.compile2();
             delegateCompiler.close();
             elapsed_msec = delegateCompiler.elapsed_msec;
@@ -963,7 +927,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                 filesSoFar.add(fileObject);
                 trees.append(parse(fileObject));
             }
-            memoryWatch.abortIfMemoryLow();
         }
         return trees.toList();
     }
@@ -1013,17 +976,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
     }
 
     private void complete(List<JCCompilationUnit> roots, ClassSymbol c) {
-        DiagnosticListener<? super JavaFileObject> oldDiagListener = log.getDiagnosticListener();
-        log.setRecordedOverlay(recordedOverlay = new HashSet<Pair<JavaFileObject, Integer>>());
-        log.setErrTreesOverlay(errTreesOverlay = new HashMap<JCTree, JCDiagnostic>());
-        log.setDiagnosticListener(temporaryDiagListener);
-        try {
-            enter.complete(roots, c);
-        } finally {
-            log.setRecordedOverlay(null);
-            log.setErrTreesOverlay(null);
-            log.setDiagnosticListener(oldDiagListener);
-        }
+        enter.complete(roots, c);
     }
 
     public void initNotYetEntered(Map<JavaFileObject, JCCompilationUnit> notYetEntered) {
@@ -1140,7 +1093,9 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                     boolean errors = false;
                     for (String nameStr : classnames) {
                         Symbol sym = resolveBinaryNameOrIdent(nameStr);
-                        if (sym == null || (sym.kind == Kinds.PCK && !processPcks)) {
+                        if (sym == null ||
+                            (sym.kind == Kinds.PCK && !processPcks) ||
+                            sym.kind == Kinds.ABSENT_TYP) {
                             log.error("proc.cant.find.class", nameStr);
                             errors = true;
                             continue;
@@ -1170,32 +1125,28 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
                     }
                 }
             }
-            try {
-                JavaCompiler c = this;
-                List<JCCompilationUnit> currentAnnotations = toProcessAnnotations.prependList(roots);
-                toProcessAnnotations = List.nil();
-                final boolean hasOrigin = currentAnnotations.nonEmpty();
-                if (hasOrigin) {
-                    fileManager.handleOption("apt-origin", Collections.singleton(currentAnnotations.head.getSourceFile().toUri().toString()).iterator());    //NOI18N
-                }
-                try {
-                    if (hasOrigin || classSymbols.nonEmpty() || pckSymbols.nonEmpty()) {
-                        c = procEnvImpl.doProcessing(context, currentAnnotations, classSymbols, pckSymbols);
-                    } //else: may happen when called from JavacTaskImpl.attribute, which forces processing of toProcessAnnotations
-                } finally {
-                    if (hasOrigin) {
-                        fileManager.handleOption("apt-origin", Collections.singletonList("").iterator());   //NOI18N
-                    }
-                }
-                if (c != this)
-                    annotationProcessingOccurred = c.annotationProcessingOccurred = true;
-                // doProcessing will have handled deferred diagnostics
-                Assert.check(c.log.deferDiagnostics == false
-                        && c.log.deferredDiagnostics.size() == 0);
-                return c;
-            } finally {
-                procEnvImpl.close(false);
+            JavaCompiler c = this;
+            List<JCCompilationUnit> currentAnnotations = toProcessAnnotations.prependList(roots);
+            toProcessAnnotations = List.nil();
+            final boolean hasOrigin = currentAnnotations.nonEmpty();
+            if (hasOrigin) {
+                fileManager.handleOption("apt-origin", Collections.singleton(currentAnnotations.head.getSourceFile().toUri().toString()).iterator());    //NOI18N
             }
+            try {
+                if (hasOrigin || classSymbols.nonEmpty() || pckSymbols.nonEmpty()) {
+                    c = procEnvImpl.doProcessing(context, currentAnnotations, classSymbols, pckSymbols);
+                } //else: may happen when called from JavacTaskImpl.attribute, which forces processing of toProcessAnnotations
+            } finally {
+                if (hasOrigin) {
+                    fileManager.handleOption("apt-origin", Collections.singletonList("").iterator());   //NOI18N
+                }
+            }
+            if (c != this)
+                annotationProcessingOccurred = c.annotationProcessingOccurred = true;
+            // doProcessing will have handled deferred diagnostics
+            Assert.check(c.log.deferDiagnostics == false
+                    && c.log.deferredDiagnostics.size() == 0, "Deferred diagnistics: " + c.log.deferredDiagnostics);
+            return c;
         } catch (CompletionFailure ex) {
             log.error("cant.access", ex.sym, ex.getDetailValue());
             log.reportDeferredDiagnostics();
@@ -1236,7 +1187,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         ListBuffer<Env<AttrContext>> results = lb();
         while (!envs.isEmpty()) {
             results.append(attribute(envs.remove()));
-            memoryWatch.abortIfMemoryLow();
         }
         return stopIfError(CompileState.ATTR, results);
     }
@@ -1286,7 +1236,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         ListBuffer<Env<AttrContext>> results = lb();
         for (Env<AttrContext> env: envs) {
             flow(env, results);
-            memoryWatch.abortIfMemoryLow();
         }
         return stopIfError(CompileState.FLOW, results);
     }
@@ -1322,13 +1271,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             try {
                 make.at(Position.FIRSTPOS);
                 TreeMaker localMake = make.forToplevel(env.toplevel);
-                if (flowListener != null) {
-                    flowListener.flowStarted (env);
-                }
                 flow.analyzeTree(env, localMake);
-                if (flowListener != null) {
-                    flowListener.flowFinished (env);
-                }
                 compileStates.put(env, CompileState.FLOW);
 
                 if (shouldStop(CompileState.FLOW))
@@ -1357,7 +1300,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         ListBuffer<Pair<Env<AttrContext>, JCClassDecl>> results = lb();
         for (Env<AttrContext> env: envs) {
             desugar(env, results);
-            memoryWatch.abortIfMemoryLow();
         }
         return stopIfError(CompileState.FLOW, results);
     }
@@ -1398,7 +1340,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             public void visitClassDef(JCClassDecl node) {
                 if (node.sym != null) {
                     Type st = types.supertype(node.sym.type);
-                    if (st.tag == TypeTags.CLASS) {
+                    if (st != null && st.tag == TypeTags.CLASS) {
                         ClassSymbol c = st.tsym.outermostClass();
                         Env<AttrContext> stEnv = enter.getEnv(c);
                         if (stEnv != null && env != stEnv) {
@@ -1550,8 +1492,6 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
             }
 
             taskFinished(new TaskEvent(TaskEvent.Kind.GENERATE, env.toplevel, cdef.sym));
-            
-            memoryWatch.abortIfMemoryLow();
         }
     }
 
@@ -1667,7 +1607,7 @@ public class JavaCompiler implements ClassReader.SourceCompleter {
         try {
             fileManager.flush();
             if (procEnvImpl != null)
-                procEnvImpl.close(true);
+                procEnvImpl.close();
             procEnvImpl = null;
         } catch (IOException e) {
             throw new Abort(e);
