@@ -811,6 +811,10 @@ public class Scanner implements Lexer {
                         putChar('0');
                         if (ch == '_') {
                             int savePos = bp;
+                            if (!allowUnderscoresInLiterals) {
+                                lexError("unsupported.underscore.lit", source.name);
+                                allowUnderscoresInLiterals = true;
+                            }
                             do {
                                 scanChar();
                             } while (ch == '_');
@@ -986,10 +990,16 @@ public class Scanner implements Lexer {
     }
 
     /** Sets the current token.
+     * This method is primarily used to update the token stream when the
+     * parser is handling the end of nested type arguments such as
+     * {@code List<List<String>>} and needs to disambiguate between
+     * repeated use of ">" and relation operators such as ">>" and ">>>". Noting
+     * that this does not handle arbitrary tokens containing Unicode escape
+     * sequences.
      */
     public void token(Token token) {
-        this.pos += this.token.name.length() - token.name.length();
-        this.prevEndPos = this.pos;
+        pos += (this.token.name != null ? this.token.name.length() : this.sp) - token.name.length();
+        prevEndPos = pos;
         this.token = token;
     }
 
