@@ -74,7 +74,6 @@ import com.sun.tools.javac.tree.*;
 import com.sun.tools.javac.tree.JCTree.*;
 import com.sun.tools.javac.util.Abort;
 import com.sun.tools.javac.util.Assert;
-import com.sun.tools.javac.util.CancelAbort;
 import com.sun.tools.javac.util.ClientCodeException;
 import com.sun.tools.javac.util.Context;
 import com.sun.tools.javac.util.Convert;
@@ -552,7 +551,7 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
                 invalid = true;
                 throw e;
             } catch (Throwable t) {
-                if (t instanceof ThreadDeath) throw (ThreadDeath) t;
+                rethrowAbort(t);
                 LOGGER.log(Level.INFO, "Annotation processing error:", t);
                 invalid = true;
             }
@@ -823,6 +822,8 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
     private boolean callProcessor(Processor proc,
                                          Set<? extends TypeElement> tes,
                                          RoundEnvironment renv) {
+        ClassLoader origContextCL = Thread.currentThread().getContextClassLoader();
+        Thread.currentThread().setContextClassLoader(proc.getClass().getClassLoader());
         try {
             return proc.process(tes, renv);
         } catch (BadClassFile ex) {
@@ -836,10 +837,11 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         } catch (ClientCodeException e) {
             throw e;
         } catch (Throwable t) {
-            if (t instanceof ThreadDeath)
-                throw (ThreadDeath)t;
+            rethrowAbort(t);
             LOGGER.log(Level.INFO, "Annotation processing error:", t);
             return false;
+        } finally {
+            Thread.currentThread().setContextClassLoader(origContextCL);
         }
     }
 
@@ -1173,19 +1175,13 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
 
         errorStatus = errorStatus || (compiler.errorCount() > 0);
 
-        // Free resources
-        this.close(false);
-
         if (taskListener != null)
             taskListener.finished(new TaskEvent(TaskEvent.Kind.ANNOTATION_PROCESSING));
 
         try {
             compiler.enterTrees(roots);
         } catch (Throwable t) {
-            if (t instanceof ThreadDeath)
-                throw (ThreadDeath)t;
-            if (t instanceof CancelAbort)
-                throw (CancelAbort)t;
+            rethrowAbort(t);
             LOGGER.log(Level.INFO, "Error while re-entering:", t);
             throw new Abort(t);
         }            
@@ -1213,22 +1209,16 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
      * Free resources related to annotation processing.
      */
     public void close() {
-        close(true);
-    }
-
-    public void close(boolean dropProcessors) {
         filer.close();
-        if (dropProcessors) {
-            if (discoveredProcs != null) // Make calling close idempotent
-                discoveredProcs.close();
-            discoveredProcs = null;
-            if (processorClassLoader != null && processorClassLoader instanceof Closeable)
-            try {
-                ((Closeable) processorClassLoader).close();
-            } catch (IOException e) {
-                JCDiagnostic msg = diags.fragment("fatal.err.cant.close.loader");
-                throw new FatalError(msg, e);
-            }
+        if (discoveredProcs != null) // Make calling close idempotent
+            discoveredProcs.close();
+        discoveredProcs = null;
+        if (processorClassLoader != null && processorClassLoader instanceof Closeable)
+        try {
+            ((Closeable) processorClassLoader).close();
+        } catch (IOException e) {
+            JCDiagnostic msg = diags.fragment("fatal.err.cant.close.loader");
+            throw new FatalError(msg, e);
         }
     }
 
@@ -1563,4 +1553,10 @@ public class JavacProcessingEnvironment implements ProcessingEnvironment, Closea
         }
         return true;
     }
+
+    private static void rethrowAbort(final Throwable t) throws Error {
+        if (t instanceof ThreadDeath || t instanceof Abort) {
+            throw (Error) t;
+        }
+    }
 }
